# ---------------------------------------------------------------------------
# HCAtoolkit.py
# Created by: Andrew Shirk
# ---------------------------------------------------------------------------

# Import system modules
import sys, string, os, arcgisscripting

# Create the Geoprocessor object
gp = arcgisscripting.create()

# Check out necessary licenses
try:
    gp.CheckOutExtension("Spatial")
except RuntimeError:
    gp.AddError("You need to have the Spatial Analyst extension to run this tool")
    sys.exit(1) # end script process


# User Inputs:

outputdir = sys.argv[1]  ### outputdir where all inputs will be located and all outputs will be written
outputfile = sys.argv[2]     ### output file name
habitat = sys.argv[3] ### binary habitat map raster grid name (habitat = 1, non-habitat =0)
resistance = sys.argv[4]      ### resistance surface raster grid
HomeRangeRadius = sys.argv[5]        ### in map units. Refers to the radius of the species' home range. Used to define the moving window size and the cost-weighted distance limit for pixels to be aggegated into a single HCA.
MinAvgHabValue = sys.argv[6]       ### the proportion of the moving window that must be habitat before the center window pixel will be considered for inclusion in an HCA
HabValueThreshold = sys.argv[7]     ### the value above which a grid cell will be considered core habitat
MinHCAarea = sys.argv[8]        ### area in map units squared. HCAs below this size will be eliminated
CalcHabitatDensity = sys.argv[9]    ### Y or N answer. Answer N if you're rerunning the model and you only change the MinAvgHabValue value
DeleteIntermediates = sys.argv[10]  ### Y or N answer.

# Set overwrite to ON
gp.Overwriteoutput = 1

# set workspace
gp.Workspace = outputdir

# Get cell size and convert home range area to number of grid cells
CellSize = gp.GetRasterProperties(habitat,"CELLSIZEX")
MinHCAcells = str(float(MinHCAarea) / (float(CellSize) * float(CellSize)))

# Calculate proportion of habitat within a HomeRangeRadius sized moving window

if CalcHabitatDensity == "true":
    gp.AddMessage ('Calculating average habitat value within home range radius moving window')
    gp.FocalStatistics_sa(habitat, "avghabvalue", "Circle " + str(float(HomeRangeRadius)) + " Map", "MEAN", "DATA")

# Set Null habitat pixels where moving window < MinAvgHabValue
gp.AddMessage ('Removing habitat where average habitat value < minimum average habitat value')
gp.SetNull_sa("avghabvalue", habitat, "habitatmask", "Value < " + MinAvgHabValue)

# Reclassify areas where set_null = 0 into NoData so that cost distance is calculated from habitat (value = 1)
gp.AddMessage ('Identifying core habitat based on habitat value threshold')
reclassifyRanges = "0 0 NoData; 0 " + str(HabValueThreshold) + " NoData; " + str(HabValueThreshold) + " 1000 1"
gp.Reclassify_sa("habitatmask", "Value", reclassifyRanges, "corehabitat", "DATA")

# For habitat cells in areas where habitat ratio is > MinAvgHabValue, run cost distance 
gp.AddMessage ('Aggregating core habitat cells within home range radius')
gp.CostDistance_sa("corehabitat", resistance, "cd_corehab", str(float(HomeRangeRadius)), "#")

# Reclassify areas where cd_corehab is < HomeRangeRadius as 1, else nodata where cd_corehab > 1
reclassifyRanges = "0.000000 " + str(float(HomeRangeRadius)) + " 1;" + str(float(HomeRangeRadius)) + ".000000001 10000000000000 0"
gp.Reclassify_sa("cd_corehab", "Value", reclassifyRanges, "prelim_hcas", "DATA")

# regiongroups cells within each prelim_hcas
gp.RegionGroup_sa("prelim_hcas", "region_group", "EIGHT", "CROSS")

# generate final raster by extracting patches that meet size requirments
gp.AddMessage ('Removing HCAs with area less than the specified minimum HCA area')
gp.SetNull_sa("region_group", "region_group", "minarea_hcas", "Count <= " + MinHCAcells)

# build raster attribute table to avoid error if too many HCAs
gp.BuildRasterAttributeTable_management("minarea_hcas","#")
 
# Region group patches
gp.AddMessage ('Generating HCA raster')
gp.RegionGroup_sa("minarea_hcas", "hca_raster" , "EIGHT", "CROSS")

# Convert HCA raster zones to polygons
gp.AddMessage ('Converting HCA raster to polygon')
gp.RasterToPolygon_conversion ("hca_raster", "prelim_hca.shp", "SIMPLIFY")
            
# Add area attribute to HCA polygons
gp.AddField_management("prelim_hca.shp", "AREA", "DOUBLE")
gp.CalculateField_management("prelim_hca.shp", "AREA", "float(!SHAPE.AREA!)","PYTHON_9.3")

# Convert prelim HCAs to layer (required to be used for 'select by attributes')
gp.MakeFeatureLayer_management("prelim_hca.shp", "TempFeatureLayer")
expression = " \"AREA\" > " + str(float(MinHCAarea)) + " "
gp.SelectLayerByAttribute_management("TempFeatureLayer", "NEW_SELECTION", expression)
gp.CopyFeatures_management("TempFeatureLayer",outputfile)

if DeleteIntermediates == "true":
    gp.delete_management("avghabvalue")
    gp.delete_management("habitatmask")
    gp.delete_management("corehabitat")
    gp.delete_management("cd_corehab")
    gp.delete_management("prelim_hcas")
    gp.delete_management("region_group")
    gp.delete_management("minarea_hcas")
    gp.delete_management("hca_raster")
    gp.delete_management("prelim_hca.shp")

else:
    gp.delete_management("region_group")
    gp.delete_management("prelim_hca.shp")
    
gp.AddMessage ('HCA processing complete')
              
